@@ -1,11 +1,13 @@
 from .prismaEnums import *
 
-from urllib.error import HTTPError
-from functools import wraps
 import requests
 import json
 import time
 
+from urllib.error import HTTPError
+from functools import wraps
+from getpass import getpass
+
 
 class PrismaSession:
     """
@@ -14,7 +16,6 @@ class PrismaSession:
     """
 
     def __init__(self, apiUrl: str, token: str, token_expiration_time: float):
-
         self.apiUrl = apiUrl
         self.token = token
         self.token_expiration_time = token_expiration_time
@@ -27,12 +28,13 @@ def refreshToken(prisma_query):
     token si quedan menos de 2 minutos para que expire.
 
     """
+
     @wraps(prisma_query)
     def wrapper(prismaSession: PrismaSession, *args, **kwargs):
         if (time.time() / 60) + 2 > prismaSession.token_expiration_time:
             prisma_extend_session(prismaSession)
-        return prisma_query(prismaSession,*args,**kwargs)
-    
+        return prisma_query(prismaSession, *args, **kwargs)
+
     return wrapper
 
 
@@ -46,6 +48,15 @@ def prisma_login(apiUrl: str, access_key_id: str, access_key_pass: str) -> Prism
     :return: JWT token válido con el que hacer llamadas a la API de prisma
     """
 
+    if apiUrl is None:
+        apiUrl = input('Prisma Cloud API URL:\n')
+    if access_key_id is None:
+        access_key_id = input('Access key ID:\n')
+    if access_key_pass is None:
+        access_key_pass = getpass('Secret key:\n')
+
+    if apiUrl.endswith('/'): apiUrl = apiUrl[:-1]
+
     payload = {
         'username': access_key_id,
         'password': access_key_pass
@@ -61,7 +72,7 @@ def prisma_login(apiUrl: str, access_key_id: str, access_key_pass: str) -> Prism
 
     if response.status_code == 200 or response.status_code == 202:
         # En la API de prisma, el tiempo de caducidad son 10 minutos, los calculamos a mano cuando llega el token
-        return PrismaSession(apiUrl, json.loads(response.text)['token'], (time.time() / 60) + 10 )
+        return PrismaSession(apiUrl, json.loads(response.text)['token'], (time.time() / 60) + 10)
 
     else:
         raise HTTPError(apiUrl + '/login', response.status_code,
@@ -100,7 +111,7 @@ def list_account_groups(prismaSession: PrismaSession, excludeCloudAccountDetails
     :param excludeCloudAccountDetails: booleano para indicar si se excluyen o no dettales de las cuentas, default a False
     :return: Nuevo JWT token con el que hacer llamadas a la API de prisma
     """
-    
+
     querystring = {
         "excludeCloudAccountDetails": excludeCloudAccountDetails
     }
@@ -120,12 +131,18 @@ def list_account_groups(prismaSession: PrismaSession, excludeCloudAccountDetails
 
 
 @refreshToken
-def list_alerts_v2(prismaSession: PrismaSession,  timeType: str = 'relative', timeAmount: str = '30', timeUnit: TimeUnit = TimeUnit.DAY, detailed: bool = True,
-                   sortBy: str = None, offset: int = 0, limit: int = 10000, pageToken: str = None, alertId: str = None, alertStatus: AlertStatus = None, 
-                   cloudAccount: str = None, cloudAccountId: str = None, accountGroup: str = None, cloudType: CloudType = None, cloudRegion: str = None, 
-                   cloudService: str = None, policyId: str = None, policyName: str = None, policySeverity: PolicySeverity = None, policyLabel: str = None, 
-                   policyType: PolicyType = None, policyComplianceStandard: str = None, policyComplianceRequirement: str = None, policyComplianceSection: str = None, 
-                   policyRemediable: bool = None, alertRuleName: str = None, resourceId: str = None, resourceName: str = None, resourceType: str = None) -> json:
+def list_alerts_v2(prismaSession: PrismaSession, timeType: str = 'relative', timeAmount: str = '30',
+                   timeUnit: TimeUnit = TimeUnit.DAY, detailed: bool = True,
+                   sortBy: str = None, offset: int = 0, limit: int = 10000, pageToken: str = None, alertId: str = None,
+                   alertStatus: AlertStatus = None,
+                   cloudAccount: str = None, cloudAccountId: str = None, accountGroup: str = None,
+                   cloudType: CloudType = None, cloudRegion: str = None,
+                   cloudService: str = None, policyId: str = None, policyName: str = None,
+                   policySeverity: PolicySeverity = None, policyLabel: str = None,
+                   policyType: PolicyType = None, policyComplianceStandard: str = None,
+                   policyComplianceRequirement: str = None, policyComplianceSection: str = None,
+                   policyRemediable: bool = None, alertRuleName: str = None, resourceId: str = None,
+                   resourceName: str = None, resourceType: str = None) -> json:
     """
     Obtiene una lista de alertas de Prisma paginada.
 
@@ -171,7 +188,6 @@ def list_alerts_v2(prismaSession: PrismaSession,  timeType: str = 'relative', ti
         "detailed": detailed,
     }
 
-
     if sortBy: querystring['sortBy'] = sortBy
     if offset < 10000 and offset > 0: querystring['offset'] = offset
     if limit: querystring['limit'] = limit
@@ -236,10 +252,14 @@ def get_alert_count_by_status(prismaSession: PrismaSession, status: AlertStatus
 
 
 @refreshToken
-def list_alert_counts_by_policy(prismaSession: PrismaSession, alertId: str = None, alertStatus: AlertStatus = None, cloudAccount: str = None, cloudAccountId: str = None,
-                                accountGroup: str = None, cloudType: str = None, cloudRegion: str = None, cloudService: str = None, policyId: str = None, policyName: str = None,
-                                policySeverity: PolicySeverity = None, policyLabel: str = None, policyType: PolicyType = None, policyComplianceStandard: str = None,
-                                policyComplianceRequirement: str = None, policyComplianceSection: str = None, policyRemediable: bool = None, alertRuleName: str = None,
+def list_alert_counts_by_policy(prismaSession: PrismaSession, alertId: str = None, alertStatus: AlertStatus = None,
+                                cloudAccount: str = None, cloudAccountId: str = None,
+                                accountGroup: str = None, cloudType: str = None, cloudRegion: str = None,
+                                cloudService: str = None, policyId: str = None, policyName: str = None,
+                                policySeverity: PolicySeverity = None, policyLabel: str = None,
+                                policyType: PolicyType = None, policyComplianceStandard: str = None,
+                                policyComplianceRequirement: str = None, policyComplianceSection: str = None,
+                                policyRemediable: bool = None, alertRuleName: str = None,
                                 resourceId: str = None, resourceName: str = None, resourceType: str = None) -> int:
     """
     Devuelve el número de alertas que surgen de cierta policy.
@@ -305,9 +325,12 @@ def list_alert_counts_by_policy(prismaSession: PrismaSession, alertId: str = Non
 
 
 @refreshToken
-def assests_inventory_view_v2(prismaSession: PrismaSession,  timeType: str = 'relative', timeAmount: str = '30', timeUnit: TimeUnit = TimeUnit.DAY,
-                              cloudAccount: str = None, accountGroup: str = None, cloudType: str = None, cloudRegion: str = None, cloudService: str = None,
-                              resourceType: str = None, groupBy: GroupBy = GroupBy.CLOUD_TYPE, scanStatus: ScanStatus = ScanStatus.ALL, policyComplianceStandard: str = None,
+def assests_inventory_view_v2(prismaSession: PrismaSession, timeType: str = 'relative', timeAmount: str = '30',
+                              timeUnit: TimeUnit = TimeUnit.DAY,
+                              cloudAccount: str = None, accountGroup: str = None, cloudType: str = None,
+                              cloudRegion: str = None, cloudService: str = None,
+                              resourceType: str = None, groupBy: GroupBy = GroupBy.CLOUD_TYPE,
+                              scanStatus: ScanStatus = ScanStatus.ALL, policyComplianceStandard: str = None,
                               policyComplianceRequirement: str = None, policyComplianceSection: str = None) -> json:
     """
     Devuelve el número de alertas que surgen de cierta policy.
@@ -363,9 +386,13 @@ def assests_inventory_view_v2(prismaSession: PrismaSession,  timeType: str = 're
 
 
 @refreshToken
-def assets_inventory_trend_view_v2(prismaSession: PrismaSession, timeType: str = 'relative', timeAmount: str = '30', timeUnit: TimeUnit = TimeUnit.DAY, cloudAccount: str = None, 
-                   accountGroup: str = None, cloudType: str = None, cloudRegion: str = None, cloudService: str = None, resourceType: str = None, scanStatus: ScanStatus = ScanStatus.ALL, 
-                   policyComplianceStandard: str = None, policyComplianceRequirement: str = None, policyComplianceSection: str = None) -> json:
+def assets_inventory_trend_view_v2(prismaSession: PrismaSession, timeType: str = 'relative', timeAmount: str = '30',
+                                   timeUnit: TimeUnit = TimeUnit.DAY, cloudAccount: str = None,
+                                   accountGroup: str = None, cloudType: str = None, cloudRegion: str = None,
+                                   cloudService: str = None, resourceType: str = None,
+                                   scanStatus: ScanStatus = ScanStatus.ALL,
+                                   policyComplianceStandard: str = None, policyComplianceRequirement: str = None,
+                                   policyComplianceSection: str = None) -> json:
     """
     Obtiene una lista de alertas de Prisma paginada.
 
@@ -408,7 +435,8 @@ def assets_inventory_trend_view_v2(prismaSession: PrismaSession, timeType: str =
         "x-redlock-auth": prismaSession.token
     }
 
-    response = requests.request("GET", prismaSession.apiUrl + '/v2/inventory/trend', params=querystring, headers=headers)
+    response = requests.request("GET", prismaSession.apiUrl + '/v2/inventory/trend', params=querystring,
+                                headers=headers)
 
     if response.status_code == 200 or response.status_code == 202:
         return json.loads(response.text)
@@ -425,7 +453,7 @@ def list_cloud_accounts(prismaSession: PrismaSession) -> json:
     :param prismaSession: datos de la sesión con el tenant (url, token)
     :return: json con cada una de las cuentas del tenant
     """
-    
+
     headers = {
         'x-redlock-auth': prismaSession.token
     }
@@ -448,7 +476,7 @@ def list_cloud_org_accounts(prismaSession: PrismaSession, cloud_type, id) -> jso
     :param id: ID de la cuenta organización
     :return: json con cada una de las cuentas hijas de la organización
     """
-    
+
     headers = {
         'x-redlock-auth': prismaSession.token
     }
@@ -461,6 +489,61 @@ def list_cloud_org_accounts(prismaSession: PrismaSession, cloud_type, id) -> jso
                         json.loads(response.text)['message'], response.headers, None)
 
 
+@refreshToken
+def list_cloud_accounts_names(prismaSession: PrismaSession, onlyActive: bool = False, accountGroupIds: list[str] = None, cloud_type: str = None) -> json:
+    """
+    Obtiene un json con todos los IDs y nombres de las cuentas del tenant pudiendo 
+    filtrarse por account group y cloud type.
+
+    :param prismaSession: datos de la sesión con el tenant (url, token)
+    :param onlyActive: Booleano para indicar si solo queremos las cuentas activas, por defecto a False.
+    :param accountGroupsIds: Lista con los IDs de los account groups que se quieren conseguir.
+    :param cloud_type: tipo de nube. Puede ser 'aws', 'azure', 'oci' o 'gpc'.
+    :return: json con todos los IDs y nombres de las cuentas
+    """
+
+    headers = {
+        'x-redlock-auth': prismaSession.token
+    }
+
+    querystring = {
+        "onlyActive": onlyActive
+    }
+
+    if accountGroupIds: querystring["accountGroupIds"] = accountGroupIds
+    if cloud_type: querystring["cloudType"] = cloud_type
+
+    response = requests.request(
+        "GET", prismaSession.apiUrl + '/cloud/name', headers=headers, params=querystring)
+    if response.status_code == 200 or response.status_code == 202:
+        return json.loads(response.text)
+    else:
+        raise HTTPError(prismaSession.apiUrl, response.status_code,
+                        json.loads(response.text), response.headers, None)
+
+
+@refreshToken
+def list_account_groups(prismaSession: PrismaSession, onlyActive: bool = False, accountGroupIds: list[str] = None, cloud_type: str = None) -> json:
+    """
+    Obtiene un json con todas las account groups (id, name y autoCreated).
+
+    :param prismaSession: datos de la sesión con el tenant (url, token)
+    :return: json con con todas las account groups.
+    """
+
+    headers = {
+        'x-redlock-auth': prismaSession.token
+    }
+
+    response = requests.request(
+        "GET", prismaSession.apiUrl + '/cloud/group/name', headers=headers)
+    if response.status_code == 200 or response.status_code == 202:
+        return json.loads(response.text)
+    else:
+        raise HTTPError(prismaSession.apiUrl, response.status_code,
+                        json.loads(response.text), response.headers, None)
+
+
 @refreshToken
 def get_standard_compliance_statistics(prismaSession: PrismaSession, standard_id) -> json:
     """
@@ -488,12 +571,14 @@ def get_standard_compliance_statistics(prismaSession: PrismaSession, standard_id
 
 
 @refreshToken
-def get_standard_compliance_statistics_trend(prismaSession: PrismaSession, standard_id) -> json:
+def get_standard_compliance_statistics_trend(prismaSession: PrismaSession, standard_id: str,
+                                             cloudAccount: str = None) -> json:
     """
     Obtiene un json con el historial de estadisticas de un standard de Prisma.
 
     :param prismaSession: datos de la sesión con el tenant (url, token)
     :param standard_id: ID del standard
+    :param cloudAccount: Nombre de la cuenta de la que se buscan las estadísticas
     :return: json con estadisticas del standard de Prisma
     """
 
@@ -504,6 +589,9 @@ def get_standard_compliance_statistics_trend(prismaSession: PrismaSession, stand
 
     querystring = {"timeType": "to_now", "timeUnit": "epoch"}
 
+    if cloudAccount:
+        querystring['cloud.account'] = cloudAccount
+
     response = requests.request(
         "GET", prismaSession.apiUrl + '/compliance/posture/trend/' + standard_id, headers=headers,
         params=querystring)
@@ -523,7 +611,7 @@ def list_compliance_standards(prismaSession: PrismaSession) -> json:
     :param prismaSession: datos de la sesión con el tenant (url, token)
     :return: json con cada uno de los standards de Prisma
     """
-   
+
     headers = {
         'x-redlock-auth': prismaSession.token
     }
@@ -545,7 +633,7 @@ def list_compliance_requirements(prismaSession: PrismaSession, standard_id) -> j
     :param standard_id: ID del compliance del standard
     :return: json con cada uno de los requerimientos del standard de Prisma
     """
-    
+
     headers = {
         'x-redlock-auth': prismaSession.token
     }
@@ -567,7 +655,7 @@ def list_compliance_sections(prismaSession: PrismaSession, requirement_id) -> js
     :param requirement_id: ID del compliance del requerimiento
     :return: json con cada uno de las secciones del requerimiento del standard de Prisma
     """
-    
+
     headers = {
         'x-redlock-auth': prismaSession.token
     }
@@ -590,9 +678,9 @@ def resource_usage_over_time(prismaSession: PrismaSession, accounts_ids, time_ra
     :param time_range: rango de tiempo. Por defecto, el rango de tiempo es desde el origen
     :return: json con el uso de recursos licenciables sobre el tiempo
     """
-    
+
     _tr = time_range
-    if(time_range == None):
+    if (time_range == None):
         _tr = {
             "type": "to_now",
             "value": "epoch"
@@ -618,7 +706,8 @@ def resource_usage_over_time(prismaSession: PrismaSession, accounts_ids, time_ra
 
 
 @refreshToken
-def usage_count_by_cloud_type(prismaSession: PrismaSession, accounts_ids, cloud_type, time_range=None, page_token=None) -> json:
+def usage_count_by_cloud_type(prismaSession: PrismaSession, accounts_ids, cloud_type, time_range=None,
+                              page_token=None) -> json:
     """
     Obtiene un json con el uso de recursos licenciables sobre el tiempo.
 
@@ -629,9 +718,9 @@ def usage_count_by_cloud_type(prismaSession: PrismaSession, accounts_ids, cloud_
     :param page_token: token de paginacion
     :return: json con el uso de recursos licenciables sobre el tiempo
     """
-    
+
     _tr = time_range
-    if(time_range == None):
+    if (time_range == None):
         _tr = {
             "type": "to_now",
             "value": "epoch"
@@ -649,7 +738,7 @@ def usage_count_by_cloud_type(prismaSession: PrismaSession, accounts_ids, cloud_
 
     querystring = {"cloud_type": cloud_type}
 
-    if(page_token != None):
+    if (page_token != None):
         payload["pageToken"] = page_token
 
     response = requests.request("POST", prismaSession.apiUrl + '/license/api/v1/usage/',
@@ -682,7 +771,7 @@ def list_policies(prismaSession: PrismaSession) -> json:
         raise HTTPError(prismaSession.apiUrl, response.status_code,
                         json.loads(response.text)['message'], response.headers, None)
 
-    
+
 @refreshToken
 def policy_details(prismaSession: PrismaSession, policyId: str) -> json:
     """
